#!/usr/bin/env python
# -*- coding: utf-8 -*-

# =========================================================================

import sys
import os, os.path
import math
import getopt

from database import *

from reportlab.platypus import (SimpleDocTemplate, Paragraph,
                                Spacer, Table, TableStyle, Frame,
                                flowables, PageBreak)
from reportlab.rl_config import defaultPageSize

from reportlab.lib.fonts import addMapping
from reportlab.lib.pagesizes import A4
from reportlab.lib.units import inch, cm, mm, pica
from reportlab.lib import styles, colors
from reportlab.lib.styles import getSampleStyleSheet, ParagraphStyle

from reportlab.platypus.paragraph import Paragraph
from reportlab.platypus.tables import *
from reportlab.platypus.doctemplate import *
from reportlab.platypus import flowables

from reportlab.graphics.barcode import *
from reportlab.graphics.barcode import code39
from reportlab.graphics.barcode.common import Codabar
from reportlab.graphics.barcode.common import I2of5

from reportlab.pdfbase import pdfmetrics
from reportlab.pdfbase.ttfonts import TTFont

# =========================================================================

class CardGenerator:

    def __init__(self, persons, format=A4):
        # persons is a list of Person instances
        # one per person

        self.persons = persons
        self.person_id = 0

        self.format = format
        self.pagewidth  = self.format[0]
        self.pageheight = self.format[1]
        self.styles = getSampleStyleSheet()

        # rows/cols of labels
        self.nbcols = 4
        self.nbrows = 10

        # label lot id 

        # page margins
        # (remains on paper)
        self.pagetop = 0*mm
        self.pagebottom = 0*mm
        self.pageleft = 0*mm
        self.pageright = 0*mm

        # external margins
        # (remains on paper)
        self.extx = 0*mm
        self.exty = 0*mm

        # internal margins
        self.intx = 3*mm
        self.inty = 3*mm

        # label width/height
        self.labelwidth  = (
            ((self.pagewidth - self.pageleft - self.pageright) 
             - self.nbcols*2*(self.extx))
            / self.nbcols )
        self.labelheight = (
            ((self.pageheight - self.pagetop - self.pagebottom) 
              - self.nbrows*2*(self.exty)) 
            / self.nbrows )


    def build_card(self, canvas, row, person):
        # person is a dictionary
        # (x0,y0) is the lower left corner (security margin excluded)
    

        x0 = (self.pageleft + self.extx)
        y0 = ( (self.nbrows - row - 1) * 
               (self.labelheight + 2*self.exty) 
               + self.pagebottom + self.exty )
        

        fontname = "Helvetica-Bold"
        fontsize = 10
        canvas.setFont(fontname, fontsize)
            
        canvas.drawCentredString(x0+self.intx + 0.5*self.labelwidth,
                                 y0+self.inty + 0.6*self.labelheight,
                                 person.name)

        canvas.drawCentredString(x0+self.intx + 0.5*self.labelwidth,
                                 y0+self.inty + 0.4*self.labelheight,
                                 "LPNHE Bibliothèque")

        canvas.drawCentredString(x0+self.intx + 0.5*self.labelwidth,
                                 y0+self.inty + 0.2*self.labelheight,
                                 person.barcode)


        fontname = "Helvetica-Bold"
        fontsize = 7
        canvas.setFont(fontname, fontsize)

        for col in xrange(1, self.nbcols):
            
            x1 = (self.pageleft + self.extx +
                  col*(self.labelwidth + 2*self.extx))

            y1 = y0

            # canvas.rect(x1, y1, self.labelwidth, self.labelheight, 
            #             stroke=1, fill=0)

            
            codebarre = I2of5(person.barcode, 
                              bearers=0, 
                              stop=1, checksum=0,
                              barHeight=35.0, barWidth = 2.5*.0075*inch,
                              quiet=0)                  

            # codebarre.drawOn(canvas, 
            #                  x1 + self.labelwidth/2. - codebarre._width/2,
            #                  y1 + self.height/2. - codebarre._height/2.)

            codebarre.drawOn(canvas, 
                             x1 + self.labelwidth/5.,
                             y1 + self.labelheight/3.25)
            
            canvas.drawCentredString(x1 + 
                                     self.labelwidth/2.,
                                     y1 +
                                     self.labelheight/2.9 +
                                     codebarre._height,
                                     "LPNHE - Bibliothèque")

            canvas.drawCentredString(x1 + 
                                     self.labelwidth/2.,
                                     y1 +
                                     self.labelheight/5.,
                                     person.barcode)

       
    def build_page(self, canvas, doc):
        canvas.saveState()

        for row in xrange(self.nbrows):
            self.build_card(canvas, row, 
                            self.persons[self.person_id])
            self.person_id += 1
            if self.person_id >= len(self.persons):
                # no more person
                return

        canvas.restoreState()


    def build(self):
        self.person_id = 0

        self.document = SimpleDocTemplate(
            filename="cards.pdf",
            pagesize=self.format,
            showBoundary=0,
            leftMargin=0,
            rigthMargin=0,
            topMargin=0,
            bottomMargin=0,
            allowSplitting=1)

        # to bypass an internal limit
        self.document._emptyPagesAllowed = 500
        
        npages = int(math.ceil(len(self.persons) / float(self.nbrows)))
        # npages = (len(self.persons) / self.nbrows) + 1
        self.story = npages * [PageBreak()]
        self.document.build(self.story, 
                            onFirstPage=self.build_page,
                            onLaterPages=self.build_page)

# =========================================================================
        
def main():

    # =====================================================================
    # Ici chargement de la liste des ouvrages
    # du catalogue consolidé avec toutes les cotes
    # et autres infos extraites + modifs manuelles
    # éventuelles.

    # Chargement depuis tous les fichiers disponibles

    # ---- parse command line -------------

    try:
        opts, args = \
        getopt.getopt(sys.argv[1:], "hv:", \
                      ["help", "verbose"])
    except getopt.GetoptError:
        # print help information and exit
        print >>sys.stderr, "error: usage: %s" % sys.argv[0]
        sys.exit(1)
        
    # print opts
    # print args

    tag = False
    label = "NOLABEL"
    force = False
    verbose = False

    for option, arg in opts:
        if option in ("-h", "--help"):
            print >>sys.stderr, "usage: %s [--tag] [--force] [--label <label>]" % sys.argv[0]
            sys.exit(0)
        if option in ("-v", "--verbose"):
            verbose = True

    ids = args[:]

    # =====================================================================
    # Next, load all persons or a selected subset

    init() # init database connexion

    ## TODO: add way to print only a subset...

    persons = list(Person.select())

    print persons

    # =====================================================================

    b = CardGenerator(persons)
    b.build()

    # =====================================================================

#--------------------------------------------------------------------------

main()

#--------------------------------------------------------------------------
